# PyBooks by Timo Kats
# version 1.0.3
# April 2021
# statement_of_equity.py

import json
import xlsxwriter

class statement_of_equity:
    def __init__(self, company_name):
        print('statement_of_equity will be added in the next update...')