# fibooks by Timo Kats
# version 1.0.3
# April 2021
# info.py

class info:
    def get_version():
        print("version:         1.0.3")
        print("documentation:   https://github.com/TimoKats/fibooks")
        print("made by:         Timo Kats")

    def secret_function():
        print('there are no easter eggs here...go away')